@@ -40,41 +40,12 @@
 #include <string.h>
 #include <glib/gi18n-lib.h>
 
-/**
- * GOFileFormatLevel:
- * @GO_FILE_FL_NONE: no name assigned, won't happen.
- * @GO_FILE_FL_WRITE_ONLY: PostScript etc, won't be remembered.
- * @GO_FILE_FL_NEW: Wb just created.
- * @GO_FILE_FL_MANUAL: Save gets punted to save as.
- * @GO_FILE_FL_MANUAL_REMEMBER: Ditto, but remember in history.
- * @GO_FILE_FL_AUTO: Save will save to this filename.
- * @GO_FILE_FL_LAST: last value, won't happen.
- **/
-
-/**
- * GOFileSaveScope:
- * @GO_FILE_SAVE_WORKBOOK: save the whole file.
- * @GO_FILE_SAVE_SHEET: save only current page.
- * @GO_FILE_SAVE_RANGE: save only a selected range.
- * @GO_FILE_SAVE_LAST: last value, won't happen.
- **/
-
-/**
- * GOFileProbeLevel:
- * @GO_FILE_PROBE_FILE_NAME: test only file name, don't read file contents.
- * @GO_FILE_PROBE_CONTENT: read the whole file if it's necessary.
- * @GO_FILE_PROBE_LAST: last value, won't happen.
- **/
-
-/**
- * GOFileOpenerClass:
- * @parent_class: parent class.
- * @can_probe: returns %TRUE if the files can be probed.
- * @probe: probes the file.
- * @open: opens and reads the file.
- *
- * File opener base class.
- **/
+enum {
+	FO_PROP_0,
+	FO_PROP_ID,
+	FO_PROP_DESCRIPTION,
+	FO_PROP_INTERACTIVE_ONLY
+};
 
 static void
 go_file_opener_init (GOFileOpener *fo)
@@ -83,6 +54,7 @@ go_file_opener_init (GOFileOpener *fo)
 	fo->description = NULL;
 	fo->probe_func = NULL;
 	fo->open_func = NULL;
+	fo->interactive_only = FALSE;
 }
 
 static void
@@ -103,6 +75,58 @@ go_file_opener_finalize (GObject *obj)
 	G_OBJECT_CLASS (g_type_class_peek (G_TYPE_OBJECT))->finalize (obj);
 }
 
+static void
+go_file_opener_set_property (GObject *object, guint property_id,
+			     GValue const *value, GParamSpec *pspec)
+{
+	GOFileOpener *fo = (GOFileOpener *)object;
+
+	switch (property_id) {
+	case FO_PROP_ID: {
+		char *s = g_value_dup_string (value);
+		g_free (fo->id);
+		fo->id = s;
+		break;
+	}
+	case FO_PROP_DESCRIPTION: {
+		char *s = g_value_dup_string (value);
+		g_free (fo->description);
+		fo->description = s;
+		break;
+	}
+	case FO_PROP_INTERACTIVE_ONLY:
+		fo->interactive_only = g_value_get_boolean (value);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+
+static void
+go_file_opener_get_property (GObject     *object,
+			     guint        property_id,
+			     GValue      *value,
+			     GParamSpec  *pspec)
+{
+	GOFileOpener *fo = GO_FILE_OPENER (object);
+
+	switch (property_id) {
+	case FO_PROP_ID:
+		g_value_set_string (value, fo->id);
+		break;
+	case FO_PROP_DESCRIPTION:
+		g_value_set_string (value, fo->description);
+		break;
+	case FO_PROP_INTERACTIVE_ONLY:
+		g_value_set_boolean (value, fo->interactive_only);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+
 static gboolean
 go_file_opener_can_probe_real (GOFileOpener const *fo, GOFileProbeLevel pl)
 {
@@ -116,7 +140,7 @@ go_file_opener_probe_real (GOFileOpener const *fo, GsfInput *input,
 	gboolean ret = FALSE;
 
 	if (fo->probe_func != NULL) {
-		ret =  fo->probe_func (fo, input, pl);
+		ret = fo->probe_func (fo, input, pl);
 		gsf_input_seek (input, 0, G_SEEK_SET);
 	}
 	return ret;
@@ -140,7 +164,41 @@ go_file_opener_open_real (GOFileOpener const *fo, gchar const *opt_enc,
 static void
 go_file_opener_class_init (GOFileOpenerClass *klass)
 {
-	G_OBJECT_CLASS (klass)->finalize = go_file_opener_finalize;
+	GObjectClass *goc = G_OBJECT_CLASS (klass);
+
+	goc->finalize = go_file_opener_finalize;
+	goc->set_property = go_file_opener_set_property;
+	goc->get_property = go_file_opener_get_property;
+
+	g_object_class_install_property
+		(goc,
+		 FO_PROP_ID,
+		 g_param_spec_string ("id",
+				      _("ID"),
+				      _("The identifier of the opener"),
+				      NULL,
+				      GSF_PARAM_STATIC |
+				      G_PARAM_READABLE));
+
+        g_object_class_install_property
+		(goc,
+		 FO_PROP_DESCRIPTION,
+		 g_param_spec_string ("description",
+				      _("Description"),
+				      _("The description of the opener"),
+				      NULL,
+				      GSF_PARAM_STATIC |
+				      G_PARAM_READWRITE));
+
+        g_object_class_install_property
+		(goc,
+		 FO_PROP_INTERACTIVE_ONLY,
+		 g_param_spec_boolean ("interactive-only",
+				      _("Interactive only"),
+				      _("TRUE if this opener requires interaction"),
+				       FALSE,
+				       GSF_PARAM_STATIC |
+				       G_PARAM_READWRITE));
 
 	klass->can_probe = go_file_opener_can_probe_real;
 	klass->probe = go_file_opener_probe_real;
@@ -154,13 +212,15 @@ GSF_CLASS (GOFileOpener, go_file_opener,
 /**
  * go_file_opener_setup:
  * @fo: Newly created GOFileOpener object
- * @id: Optional ID of the opener (or NULL)
+ * @id: (allow-none): ID of the opener
  * @description: Description of supported file format
- * @suffixes: (element-type char): List of suffixes to associate with the opener
- * @mimes: (element-type char): List of mime types to associate with the opener
+ * @suffixes: (element-type utf8) (transfer full): List of suffixes to
+ * associate with the opener
+ * @mimes: (element-type utf8) (transfer full): List of mime types to
+ * associate with the opener
  * @encoding_dependent: whether the opener depends on an encoding sel.
- * @probe_func: (scope async):  Optional pointer to "probe" function (or NULL)
- * @open_func: (scope async): Pointer to "open" function
+ * @probe_func: (scope async) (nullable): "probe" function
+ * @open_func: (scope async): "open" function
  *
  * Sets up GOFileOpener object, newly created with g_object_new function.
  * This is intended to be used only by GOFileOpener derivates.
@@ -176,6 +236,7 @@ go_file_opener_setup (GOFileOpener *fo, gchar const *id,
 		      GOFileOpenerOpenFunc open_func)
 {
 	g_return_if_fail (GO_IS_FILE_OPENER (fo));
+	g_return_if_fail (description != NULL);
 
 	fo->id = g_strdup (id);
 	fo->description = g_strdup (description);
@@ -189,12 +250,14 @@ go_file_opener_setup (GOFileOpener *fo, gchar const *id,
 
 /**
  * go_file_opener_new:
- * @id: Optional ID of the opener (or NULL)
+ * @id: (nullable): Optional ID of the opener
  * @description: Description of supported file format
- * @suffixes: (element-type char): List of suffixes to associate with the opener
- * @mimes: (element-type char): List of mime types to associate with the opener
- * @probe_func: (scope async): Optional pointer to "probe" function (or NULL)
- * @open_func: (scope async): Pointer to "open" function
+ * @suffixes: (element-type utf8) (transfer full): List of suffixes to
+ * associate with the opener
+ * @mimes: (element-type utf8) (transfer full): List of mime types to
+ * associate with the opener
+ * @probe_func: (scope async) (nullable): "probe" function
+ * @open_func: (scope async): "open" function
  *
  * Creates new GOFileOpener object. Optional @id will be used
  * after registering it with go_file_opener_register function.
@@ -220,14 +283,16 @@ go_file_opener_new (gchar const *id,
 
 /**
  * go_file_opener_new_with_enc:
- * @id: Optional ID of the opener (or NULL)
+ * @id: (nullable): Optional ID of the opener
  * @description: Description of supported file format
- * @suffixes: (element-type char): List of suffixes to associate with the opener
- * @mimes: (element-type char): List of mime types to associate with the opener
- * @probe_func: (scope async): Optional pointer to "probe" function (or NULL)
- * @open_func: (scope async): Pointer to "open" function
+ * @suffixes: (element-type utf8) (transfer full): List of suffixes to
+ * associate with the opener
+ * @mimes: (element-type utf8) (transfer full): List of mime types to
+ * associate with the opener
+ * @probe_func: (scope async) (nullable): "probe" function
+ * @open_func: (scope async): "open" function
  *
- * Creates new GOFileOpener object. Optional @id will be used
+ * Creates new #GOFileOpener object. Optional @id will be used
  * after registering it with go_file_opener_register function.
  *
  * Returns: (transfer full): newly created #GOFileOpener object.
@@ -250,7 +315,12 @@ go_file_opener_new_with_enc (gchar const *id,
 
 
 
-
+/**
+ * go_file_opener_get_id:
+ * @fo: #GOFileOpener to query
+ *
+ * Returns: (transfer none) (nullable): the id of @fo
+ */
 gchar const *
 go_file_opener_get_id (GOFileOpener const *fo)
 {
@@ -259,6 +329,12 @@ go_file_opener_get_id (GOFileOpener const *fo)
 	return fo->id;
 }
 
+/**
+ * go_file_opener_get_description:
+ * @fo: #GOFileOpener to query
+ *
+ * Returns: (transfer none): the description of @fo
+ */
 gchar const *
 go_file_opener_get_description (GOFileOpener const *fo)
 {
@@ -267,6 +343,12 @@ go_file_opener_get_description (GOFileOpener const *fo)
 	return fo->description;
 }
 
+/**
+ * go_file_opener_is_encoding_dependent:
+ * @fo: #GOFileOpener to query
+ *
+ * Returns: %TRUE if @fo is encoding dependent
+ */
 gboolean
 go_file_opener_is_encoding_dependent (GOFileOpener const *fo)
 {
@@ -275,6 +357,13 @@ go_file_opener_is_encoding_dependent (GOFileOpener const *fo)
 	return fo->encoding_dependent;
 }
 
+/**
+ * go_file_opener_can_probe:
+ * @fo: #GOFileOpener to query
+ * @pl: probe level
+ *
+ * Returns: %TRUE if @fo has a probe function
+ */
 gboolean
 go_file_opener_can_probe (GOFileOpener const *fo, GOFileProbeLevel pl)
 {
@@ -287,7 +376,8 @@ go_file_opener_can_probe (GOFileOpener const *fo, GOFileProbeLevel pl)
  * go_file_opener_get_suffixes:
  * @fo: #GOFileOpener
  *
- * Returns: (element-type char) (transfer none): the suffixes for the supporte file types.
+ * Returns: (element-type utf8) (transfer none): the suffixes for the
+ * supported file types.
  **/
 GSList const *
 go_file_opener_get_suffixes (GOFileOpener const *fo)
@@ -298,9 +388,9 @@ go_file_opener_get_suffixes (GOFileOpener const *fo)
 
 /**
  * go_file_opener_get_mimes:
- * @fo: #GOFileOpener
+ * @fo: #GOFileOpener to query
  *
- * Returns: (element-type char) (transfer none): the supported mime types.
+ * Returns: (element-type utf8) (transfer none): the supported mime types.
  **/
 GSList const *
 go_file_opener_get_mimes (GOFileOpener const *fo)
@@ -337,7 +427,7 @@ go_file_opener_probe (GOFileOpener const *fo, GsfInput *input, GOFileProbeLevel
 /**
  * go_file_opener_open:
  * @fo: GOFileOpener object
- * @opt_enc: Optional encoding
+ * @opt_enc: (nullable): Optional encoding
  * @io_context: Context for i/o operation
  * @view: #GoView
  * @input: Gsf input stream
@@ -351,7 +441,7 @@ go_file_opener_probe (GOFileOpener const *fo, GsfInput *input, GOFileProbeLevel
 void
 go_file_opener_open (GOFileOpener const *fo, gchar const *opt_enc,
 		     GOIOContext *io_context,
-		      GoView *view, GsfInput *input)
+		     GoView *view, GsfInput *input)
 {
 	g_return_if_fail (GO_IS_FILE_OPENER (fo));
 	g_return_if_fail (GSF_IS_INPUT (input));
@@ -380,6 +470,7 @@ go_file_saver_init (GOFileSaver *fs)
 	fs->mime_type = NULL;
 	fs->description = NULL;
 	fs->overwrite_files = TRUE;
+	fs->interactive_only = FALSE;
 	fs->format_level = GO_FILE_FL_NEW;
 	fs->save_scope = GO_FILE_SAVE_WORKBOOK;
 	fs->save_func = NULL;
@@ -408,6 +499,7 @@ enum {
 	FS_PROP_EXTENSION,
 	FS_PROP_DESCRIPTION,
 	FS_PROP_OVERWRITE,
+	FS_PROP_INTERACTIVE_ONLY,
 	FS_PROP_FORMAT_LEVEL,
 	FS_PROP_SCOPE
 };
@@ -427,25 +519,25 @@ go_file_saver_set_property (GObject *object, guint property_id,
 
 	switch (property_id) {
 	case FS_PROP_ID: {
-		char *s = g_strdup (g_value_get_string (value));
+		char *s = g_value_dup_string (value);
 		g_free (fs->id);
 		fs->id = s;
 		break;
 	}
 	case FS_PROP_MIME_TYPE: {
-		char *s = g_strdup (g_value_get_string (value));
+		char *s = g_value_dup_string (value);
 		g_free (fs->mime_type);
 		fs->mime_type = s;
 		break;
 	}
 	case FS_PROP_EXTENSION: {
-		char *s = g_strdup (g_value_get_string (value));
+		char *s = g_value_dup_string (value);
 		g_free (fs->extension);
 		fs->extension = s;
 		break;
 	}
 	case FS_PROP_DESCRIPTION: {
-		char *s = g_strdup (g_value_get_string (value));
+		char *s = g_value_dup_string (value);
 		g_free (fs->description);
 		fs->description = s;
 		break;
@@ -453,6 +545,9 @@ go_file_saver_set_property (GObject *object, guint property_id,
 	case FS_PROP_OVERWRITE:
 		fs->overwrite_files = g_value_get_boolean (value);
 		break;
+	case FS_PROP_INTERACTIVE_ONLY:
+		fs->interactive_only = g_value_get_boolean (value);
+		break;
 	case FS_PROP_FORMAT_LEVEL:
 		fs->format_level = g_value_get_enum (value);
 		break;
@@ -487,6 +582,9 @@ go_file_saver_get_property (GObject *object, guint property_id,
 	case FS_PROP_OVERWRITE:
 		g_value_set_boolean (value, fs->overwrite_files);
 		break;
+	case FS_PROP_INTERACTIVE_ONLY:
+		g_value_set_boolean (value, fs->interactive_only);
+		break;
 	case FS_PROP_FORMAT_LEVEL:
 		g_value_set_enum (value, fs->format_level);
 		break;
@@ -571,6 +669,16 @@ go_file_saver_class_init (GOFileSaverClass *klass)
 				       GSF_PARAM_STATIC |
 				       G_PARAM_READWRITE));
 
+        g_object_class_install_property
+		(goc,
+		 FS_PROP_INTERACTIVE_ONLY,
+		 g_param_spec_boolean ("interactive-only",
+				      _("Interactive only"),
+				      _("TRUE if this saver requires interaction"),
+				       FALSE,
+				       GSF_PARAM_STATIC |
+				       G_PARAM_READWRITE));
+
 	/* What a load of crap! */
         g_object_class_install_property
 		(goc,
@@ -614,17 +722,17 @@ GSF_CLASS (GOFileSaver, go_file_saver,
 
 /**
  * go_file_saver_new:
- * @id: Optional ID of the saver (or NULL)
- * @extension: Optional default extension of saved files (or NULL)
+ * @id: (nullable): ID of the saver
+ * @extension: (nullable): Default extension of saved files
  * @description: Description of supported file format
  * @level: File format level
  * @save_func: (scope async): Pointer to "save" function
  *
- * Creates new GOFileSaver object. Optional @id will be used
+ * Creates new #GOFileSaver object. Optional @id will be used
  * after registering it with go_file_saver_register or
  * go_file_saver_register_as_default function.
  *
- * Returns: newly created GOFileSaver object.
+ * Returns: newly created #GOFileSaver object.
  */
 GOFileSaver *
 go_file_saver_new (gchar const *id,
@@ -635,6 +743,8 @@ go_file_saver_new (gchar const *id,
 {
 	GOFileSaver *fs;
 
+	g_return_val_if_fail (description != NULL, NULL);
+
 	fs = GO_FILE_SAVER (g_object_new (GO_TYPE_FILE_SAVER,
 					  "id", id,
 					  "extension", extension,
@@ -655,6 +765,12 @@ go_file_saver_set_save_scope (GOFileSaver *fs, GOFileSaveScope scope)
 	fs->save_scope = scope;
 }
 
+/**
+ * go_file_saver_get_save_scope:
+ * @fs: #GOFileSaver to query
+ *
+ * Returns: The save scope of @fs.
+ */
 GOFileSaveScope
 go_file_saver_get_save_scope (GOFileSaver const *fs)
 {
@@ -663,6 +779,12 @@ go_file_saver_get_save_scope (GOFileSaver const *fs)
 	return fs->save_scope;
 }
 
+/**
+ * go_file_saver_get_id:
+ * @fs: #GOFileSaver to query
+ *
+ * Returns: (nullable) (transfer none): The id of @fs.
+ */
 gchar const *
 go_file_saver_get_id (GOFileSaver const *fs)
 {
@@ -671,6 +793,12 @@ go_file_saver_get_id (GOFileSaver const *fs)
 	return fs->id;
 }
 
+/**
+ * go_file_saver_get_mime_type:
+ * @fs: #GOFileSaver to query
+ *
+ * Returns: (nullable) (transfer none): The mime type of the @fs.
+ */
 gchar const *
 go_file_saver_get_mime_type (GOFileSaver const *fs)
 {
@@ -679,6 +807,13 @@ go_file_saver_get_mime_type (GOFileSaver const *fs)
 	return fs->mime_type;
 }
 
+/**
+ * go_file_saver_get_extension:
+ * @fs: #GOFileSaver to query
+ *
+ * Returns: (nullable) (transfer none): The default extensions for files saved
+ * by @fs.
+ */
 gchar const *
 go_file_saver_get_extension (GOFileSaver const *fs)
 {
@@ -687,6 +822,12 @@ go_file_saver_get_extension (GOFileSaver const *fs)
 	return fs->extension;
 }
 
+/**
+ * go_file_saver_get_description:
+ * @fs: #GOFileSaver to query
+ *
+ * Returns: (transfer none): The description of @fs.
+ */
 gchar const *
 go_file_saver_get_description (GOFileSaver const *fs)
 {
@@ -695,6 +836,12 @@ go_file_saver_get_description (GOFileSaver const *fs)
 	return fs->description;
 }
 
+/**
+ * go_file_saver_get_format_level:
+ * @fs: #GOFileSaver to query
+ *
+ * Returns: The format level of @fs.
+ */
 GOFileFormatLevel
 go_file_saver_get_format_level (GOFileSaver const *fs)
 {
@@ -774,7 +921,7 @@ go_file_saver_save (GOFileSaver const *fs, GOIOContext *io_context,
 
 /**
  * go_file_saver_set_overwrite_files:
- * @fs: GOFileSaver object
+ * @fs: #GOFileSaver object
  * @overwrite: A boolean value saying whether the saver should overwrite
  *                existing files.
  *
@@ -892,8 +1039,8 @@ go_file_opener_register (GOFileOpener *fo, gint priority)
 
 /**
  * go_file_opener_unregister:
- * @fo: GOFileOpener object previously registered using
- *                go_file_opener_register
+ * @fo: #GOFileOpener object previously registered using
+ *    go_file_opener_register
  *
  * Removes @fo opener from list of available file openers. Reference count
  * for the opener is decremented inside the function.
@@ -938,7 +1085,7 @@ default_file_saver_cmp_priority (gconstpointer a, gconstpointer b)
 
 /**
  * go_file_saver_register:
- * @fs: GOFileSaver object
+ * @fs: #GOFileSaver object
  *
  * Adds @fs saver to the list of available file savers, making it
  * available for the user when selecting file format for save.
@@ -993,7 +1140,7 @@ go_file_saver_register_as_default (GOFileSaver *fs, gint priority)
 
 /**
  * go_file_saver_unregister:
- * @fs: GOFileSaver object previously registered using
+ * @fs: #GOFileSaver object previously registered using
  *                go_file_saver_register or go_file_saver_register_as_default
  *
  * Removes @fs saver from list of available file savers. Reference count
@@ -1039,8 +1186,8 @@ go_file_saver_unregister (GOFileSaver *fs)
  * Finds file saver registered as default saver with the highest priority.
  * Reference count for the saver is NOT incremented.
  *
- * Returns: (transfer none): GOFileSaver object or NULL if default saver is not
- *               available.
+ * Returns: (transfer none) (nullable): #GOFileSaver for the highest priority
+ * default saver.
  **/
 GOFileSaver *
 go_file_saver_get_default (void)
@@ -1055,7 +1202,8 @@ go_file_saver_get_default (void)
  * go_file_saver_for_mime_type:
  * @mime_type: A mime type
  *
- * Returns: (transfer none): A #GOFileSaver object associated with @mime_type, or %NULL
+ * Returns: (transfer none) (nullable): A #GOFileSaver object associated with
+ * @mime_type.
  **/
 GOFileSaver *
 go_file_saver_for_mime_type (gchar const *mime_type)
@@ -1086,10 +1234,10 @@ go_file_saver_for_mime_type (gchar const *mime_type)
  * go_file_saver_for_file_name:
  * @file_name: name
  *
- * Searches for file saver with given @filename, registered using
- * go_file_opener_register
+ * Searches for file saver matching the given @file_name, registered using
+ * go_file_saver_register.
  *
- * Returns: (transfer none): GOFileSaver object or NULL if opener cannot be found.
+ * Returns: (transfer none) (nullable): #GOFileSaver for @file_name
  **/
 GOFileSaver *
 go_file_saver_for_file_name (char const *file_name)
@@ -1114,7 +1262,7 @@ go_file_saver_for_file_name (char const *file_name)
  * Searches for file opener with given @id, registered using
  * go_file_opener_register
  *
- * Returns: (transfer none): #GOFileOpener object or %NULL if opener cannot be found.
+ * Returns: (transfer none) (nullable): #GOFileOpener with given id.
  **/
 GOFileOpener *
 go_file_opener_for_id (gchar const *id)
@@ -1133,7 +1281,7 @@ go_file_opener_for_id (gchar const *id)
  * Searches for file saver with given @id, registered using
  * go_file_saver_register or register_file_opener_as_default.
  *
- * Returns: (transfer none): GOFileSaver object or NULL if saver cannot be found.
+ * Returns: (transfer none) (nullable): #GOFileSaver with given id.
  **/
 GOFileSaver *
 go_file_saver_for_id (gchar const *id)
@@ -1160,7 +1308,7 @@ go_get_file_savers (void)
 /**
  * go_get_file_openers:
  *
- * Returns: (element-type GOFileSaver) (transfer none): list of known
+ * Returns: (element-type GOFileOpener) (transfer none): list of known
  * #GOFileOpener types (do not modify list)
  **/
 GList *
