@@ -129,7 +129,7 @@ _go_math_init (void)
 #error "Please do not compile this code with -ffast-math.  It will not work."
 #endif
 
-	go_nan = go_pinf * 0.0;
+	go_nan = fabs (go_pinf * 0.0);
 	if (isnan (go_nan))
 		goto have_nan;
 
@@ -990,7 +990,16 @@ double
 go_sinpi (double x)
 {
 	int k;
+	double x0 = x;
 	x = reduce_half (x, &k);
+
+	/*
+	 * Per IEEE 754 2008:
+	 * sinpi(n) == 0 with sign of n.
+	 */
+	if (x == 0 && (k & 1) == 0)
+		return copysign (0, x0);
+
 	return do_sinpi (x, k);
 }
 
@@ -1006,6 +1015,14 @@ go_cospi (double x)
 {
 	int k;
 	x = reduce_half (x, &k);
+
+	/*
+	 * Per IEEE 754 2008:
+	 * cospi(n+0.5) == +0 for any integer n.
+	 */
+	if (x == 0 && (k & 1) == 1)
+		return +0.0;
+
 	return do_sinpi (x, k + 1);
 }
 
@@ -1019,29 +1036,58 @@ go_cospi (double x)
 double
 go_tanpi (double x)
 {
-	int k;
-	x = reduce_half (x, &k);
-	return do_sinpi (x, k) / do_sinpi (x, k + 1);
+	/*
+	 * IEEE 754 2008 doesn't have tanpi and thus doesn't define the
+	 * behaviour for -0 argument or result.  crlibm has tanpi, but
+	 * doesn't seem to be fully clear on these cases.
+	 */
+
+	/* inf -> nan; -n -> -0; +n -> +0 */
+	x = fmod (x, 1.0);
+
+	if (x == 0)
+		return copysign (0.0, x);
+	if (fabs (x) == 0.5)
+		return copysign (go_nan, x);
+	else
+		return go_sinpi (x) / go_cospi (x);
+}
+
+double
+go_cotpi (double x)
+{
+	/*
+	 * IEEE 754 2008 doesn't have tanpi.  Neither does crlibm.  Mirror
+	 * tanpi here.
+	 */
+
+	/* inf -> nan; -n -> -0; +n -> +0 */
+	x = fmod (x, 1.0);
+
+	if (x == 0)
+		return copysign (go_nan, x);
+	if (fabs (x) == 0.5)
+		return copysign (0.0, x);
+	else
+		return go_cospi (x) / go_sinpi (x);
 }
 
 double
 go_atan2pi (double y, double x)
 {
-	/* This ignores loads of cases with -0. */
-
-	if (y == 0)
-		return (x >= 0 ? 0 : +1);
-
-	if (x == 0)
-		return (y >= 0 ? 0.5 : -0.5);
-
-	if (fabs (x) == fabs (y))
-		return (y > 0 ? +1 : -1) * (x > 0 ? 0.25 : 0.75);
-
-	/* Fallback.  */
+	/*
+	 * This works perfectly for results that are (n/4).   We currently have
+	 * nothing interesting to add.
+	 */
 	return atan2 (y, x) / M_PI;
 }
 
+double
+go_atanpi (double x)
+{
+	return x < 0 ? -go_atan2pi (-x, 1) : go_atan2pi (x, 1);
+}
+
 #ifdef GOFFICE_WITH_LONG_DOUBLE
 
 #define M_PIgo    3.141592653589793238462643383279502884197169399375105820974944592307816406286208998628034825342117L
@@ -1080,7 +1126,7 @@ do_sinpil (long double x, int k)
 	else if (x == 0.25)
 		y = 0.707106781186547524400844362104849039284835937688474036588339L;
 	else
-		y = (k & 1) ? cos (M_PIgo * x) : sin (M_PIgo * x);
+		y = (k & 1) ? cosl (M_PIgo * x) : sinl (M_PIgo * x);
 
 	return (k & 2) ? 0 - y : y;
 }
@@ -1097,7 +1143,16 @@ long double
 go_sinpil (long double x)
 {
 	int k;
+	long double x0 = x;
 	x = reduce_halfl (x, &k);
+
+	/*
+	 * Per IEEE 754 2008:
+	 * sinpi(n) == 0 with sign of n.
+	 */
+	if (x == 0 && (k & 1) == 0)
+		return copysign (0, x0);
+
 	return do_sinpil (x, k);
 }
 
@@ -1113,6 +1168,14 @@ go_cospil (long double x)
 {
 	int k;
 	x = reduce_halfl (x, &k);
+
+	/*
+	 * Per IEEE 754 2008:
+	 * cospi(n+0.5) == +0 for any integer n.
+	 */
+	if (x == 0 && (k & 1) == 1)
+		return +0.0l;
+
 	return do_sinpil (x, k + 1);
 }
 
@@ -1126,29 +1189,58 @@ go_cospil (long double x)
 long double
 go_tanpil (long double x)
 {
-	int k;
-	x = reduce_halfl (x, &k);
-	return do_sinpil (x, k) / do_sinpil (x, k + 1);
+	/*
+	 * IEEE 754 2008 doesn't have tanpi and thus doesn't define the
+	 * behaviour for -0 argument or result.  crlibm has tanpi, but
+	 * doesn't seem to be fully clear on these cases.
+	 */
+
+	/* inf -> nan; -n -> -0; +n -> +0 */
+	x = fmodl (x, 1.0l);
+
+	if (x == 0)
+		return copysignl (0.0l, x);
+	if (fabsl (x) == 0.5l)
+		return copysignl (go_nanl, x);
+	else
+		return go_sinpil (x) / go_cospil (x);
+}
+
+long double
+go_cotpil (long double x)
+{
+	/*
+	 * IEEE 754 2008 doesn't have tanpi.  Neither does crlibm.  Mirror
+	 * tanpi here.
+	 */
+
+	/* inf -> nan; -n -> -0; +n -> +0 */
+	x = fmodl (x, 1.0l);
+
+	if (x == 0)
+		return copysignl (go_nanl, x);
+	if (fabsl (x) == 0.5)
+		return copysignl (0.0l, x);
+	else
+		return go_cospil (x) / go_sinpil (x);
 }
 
 long double
 go_atan2pil (long double y, long double x)
 {
-	/* This ignores loads of cases with -0. */
-
-	if (y == 0)
-		return (x >= 0 ? 0 : +1);
-
-	if (x == 0)
-		return (y >= 0 ? 0.5 : -0.5);
-
-	if (fabs (x) == fabs (y))
-		return (y > 0 ? +1 : -1) * (x > 0 ? 0.25 : 0.75);
-
-	/* Fallback.  */
+	/*
+	 * This works perfectly for results that are (n/4), at least with x86
+	 * long double.   We currently have nothing interesting to add.
+	 */
 	return atan2l (y, x) / M_PIgo;
 }
 
+long double
+go_atanpil (long double x)
+{
+	return x < 0 ? -go_atan2pil (-x, 1) : go_atan2pil (x, 1);
+}
+
 #endif
 
 /* ------------------------------------------------------------------------- */
